# -*- coding: utf-8 -*-

################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther
################################################################

from StringIO import StringIO
from config import PROJECTNAME, DEPENDENCIES
from Products.CMFCore.utils import getToolByName

class Generator(object):

    def installProducts(self, p, out):
        """QuickInstaller install of required Products"""
        qi = getToolByName(p, 'portal_quickinstaller')
        for product in DEPENDENCIES:
            if qi.isProductInstalled(product):
                qi.reinstallProducts([product])
            else:
                qi.installProduct(product, locked=0)
                print >> out, "Product installed: %s \n" %product

    def setCalendarProperties(self, p, out):
        tool = getToolByName(p , 'portal_calendar')
        types = list(tool.getCalendarTypes())
        for id in ('VSEvent', 'VSSubEvent'):
            if not id in types:
                types.append(id)
        tool.calendar_types = tuple(types)
        tool.manage_addProperty('vs_event_supplementary_events', True, 'boolean')
        print >> out, "VSEvent types for Calendar activated \n" 


def setupVarious(context):

    if context.readDataFile('vs.event_various.txt') is None:
        return
    # Add additional setup code here
    out = StringIO()
    site = context.getSite()
    gen = Generator()
    gen.installProducts(site, out)
    gen.setCalendarProperties(site, out)
    logger = context.getLogger(PROJECTNAME)
    logger.info(out.getvalue())



# -*- coding: iso-8859-15 -*-

################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther 
################################################################

import time
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Products.Archetypes.public import CalendarWidget
from Products.Archetypes.Registry import registerWidget

class VSCalendarWidget(CalendarWidget):
    """ A widget für URLFields """

    _properties = CalendarWidget._properties.copy()
    _properties.update({
        'macro' : "vs_calendarwidget",
        'dateformat' : '%d.%m.%Y',
        'with_time' : False,
        'default_hour' : 0,
        'default_minute' : 0,
        })

    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None, emptyReturnsMarker=False):
        fieldname = field.getName()
        datestr = form.get(fieldname)
        if not datestr:
            return empty_marker
        date_format = form.get(fieldname + '-date-format')
        hours = int(form.get(fieldname + '_hour', 0))
        minutes = int(form.get(fieldname + '_minute', 0))
        datestr = '%s %s:%s:00' % (datestr, hours, minutes)
        tp = time.strptime(datestr, date_format + ' %H:%M:%S')
        return DateTime(time.mktime(tp)), {}

InitializeClass(VSCalendarWidget)

registerWidget(VSCalendarWidget,
               title='CalendarWidget',
               description='A simple calendar widget',
               used_for=('Products.Archetypes.Field.DateTimeField',)
               )




##parameters=language=None

if not language:
    language = context.REQUEST.LANGUAGE

european_fmt = dict(python_fmt='%d.%m.%Y',
                    jquery_fmt='dd.mm.yy')

us_fmt = dict(python_fmt='%Y/%m/%d',
                jquery_fmt='yy/mm/dd')

language_mapping = {
    'de' : european_fmt,
    'at' : european_fmt,
    'ch' : european_fmt,
    'en' : us_fmt,
}

return language_mapping.get(language, us_fmt)




################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther
################################################################


"""
Calendar utilities

Doublecheck conversions:

    >>> brt = gettz('Brazil/East')

    >>> dt = DateTime('2005/07/07 18:00:00 Brazil/East')
    >>> dt == dt2DT(DT2dt(dt))
    True

    >>> dt = datetime.datetime(2005, 07, 07, 18, 00, 00, tzinfo=brt)
    >>> dt == DT2dt(dt2DT(dt))
    True

This code is taken from p4a.common
"""

import datetime
from DateTime import DateTime
from dateutil import tz

def gettz(name=None):
    try:
        return _extra_times[name]
    except KeyError:
        return tz.gettz(name)

def dt2DT(dt, tzname=None):
    """Convert a python datetime to DateTime. 

    >>> import time, os
    >>> oldtz = os.environ.get('TZ')
    >>> os.environ['TZ'] = 'Brazil/East'
    >>> time.tzset()
    >>> brt = gettz('Brazil/East')

    No timezone information, assume local timezone at the time.

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0))
    DateTime('2005/11/07 18:00:00 GMT-2')

    Provide a default TZID:

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0), tzname='EET')
    DateTime('2005/11/07 18:00:00 GMT+2')

    UTC timezone.

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0, tzinfo=gettz('UTC')))
    DateTime('2005/11/07 18:00:00 GMT+0')

    BRST timezone (GMT-2 on this day).

    >>> dt2DT(datetime.datetime(2005, 11, 07, 18, 0, 0, tzinfo=brt))
    DateTime('2005/11/07 18:00:00 GMT-2')

    BRT timezone (GMT-3 on this day).

    >>> dt2DT(datetime.datetime(2005, 07, 07, 18, 0, 0, tzinfo=brt))
    DateTime('2005/07/07 18:00:00 GMT-3')
    
    Change back:
    >>> if oldtz is None:
    ...     del os.environ['TZ']
    ... else:
    ...     os.environ['TZ'] = oldtz
    >>> time.tzset()    

    """
    if tzname is None and dt.tzinfo is None:
        # Assume local time
        timezone = gettz()
    elif tzname is not None:
        # Convert to timezone
        timezone = gettz(tzname)
    else:
        timezone = None
    if timezone is not None:
        dt = dt.replace(tzinfo=timezone)
    return DateTime(dt.isoformat())

def DT2dt(dt):
    """Convert a DateTime to python's datetime in UTC.

    >>> dt = DT2dt(DateTime('2005/11/07 18:00:00 UTC'))
    >>> dt
    datetime.datetime(2005, 11, 7, 18, 0, tzinfo=tzfile('/usr/share/zoneinfo/Universal'))
    >>> dt.astimezone(gettz('UTC'))
    datetime.datetime(2005, 11, 7, 18, 0, tzinfo=tzfile('/usr/share/zoneinfo/UTC'))

    >>> dt = DT2dt(DateTime('2005/11/07 18:00:00 Brazil/East'))
    >>> dt
    datetime.datetime(2005, 11, 7, 18, 0, tzinfo=tzfile('/usr/share/zoneinfo/Brazil/East'))
    >>> dt.astimezone(gettz('UTC'))
    datetime.datetime(2005, 11, 7, 20, 0, tzinfo=tzfile('/usr/share/zoneinfo/UTC'))

    >>> dt = DT2dt(DateTime('2005/11/07 18:00:00 GMT-2'))
    >>> dt
    datetime.datetime(2005, 11, 7, 18, 0, tzinfo=tzoffset('GMT-2', -7200))
    >>> dt.astimezone(gettz('UTC'))
    datetime.datetime(2005, 11, 7, 20, 0, tzinfo=tzfile('/usr/share/zoneinfo/UTC'))

    >>> dt = DT2dt(DateTime('2005/07/07 18:00:00 Brazil/East'))
    >>> dt
    datetime.datetime(2005, 7, 7, 18, 0, tzinfo=tzfile('/usr/share/zoneinfo/Brazil/East'))
    >>> dt.astimezone(gettz('UTC'))
    datetime.datetime(2005, 7, 7, 21, 0, tzinfo=tzfile('/usr/share/zoneinfo/UTC'))

    >>> dt = DT2dt(DateTime('2005/07/07 18:00:00 GMT-3'))
    >>> dt
    datetime.datetime(2005, 7, 7, 18, 0, tzinfo=tzoffset('GMT-3', -10800))
    >>> dt.astimezone(gettz('UTC'))
    datetime.datetime(2005, 7, 7, 21, 0, tzinfo=tzfile('/usr/share/zoneinfo/UTC'))
    """
    timezone = gettz(dt.timezone())
    value = datetime.datetime(dt.year(), dt.month(), dt.day(),
                              dt.hour(), dt.minute(), int(dt.second()),
                              int(dt.second()*1000000) % 1000000, tzinfo=timezone)
    return value

_extra_times = {}
for x in range(-12, 0) + range(1, 13):
    for n in ('GMT', 'UTC'):
        name = '%s%+i' % (n, x)
        timezone = tz.tzoffset(name, x*3600)
        _extra_times[name] = timezone



# -*- coding: utf-8 -*-
# $Id: recurrence.py 265 2009-06-10 07:47:58Z ajung $

import datetime
from dateutil import rrule
from zope import interface
from zope import component
from zope import schema
from zope.app.annotation import interfaces as annointerfaces
from dateable import kalends
from dateutil.rrule import MO, TU, WE, TH, FR, SA, SU
from dtutils import DT2dt

from vs.event.content.event import VSEvent


wdays = [MO, TU, WE, TH, FR, SA, SU]

class IRecurrenceConfig(interface.Interface):
    """Configuration information for an event.
    """
    
    is_recurring = schema.Bool(
        title=u'Recurring event',
        description=u'This event recurs'
        )

class VSRecurrenceSupport(object):
    """Recurrence support"""
    
    interface.implements(kalends.IRecurrence)
    component.adapts(VSEvent)
    
    def __init__(self, context):
        self.context = context

    def getRecurrenceRule(self):
        """Returns a dateutil.rrule"""
        if getattr(self.context, 'frequency', -1) is -1:
            return None
        
        dtstart = DT2dt(self.context.startDate)
        if self.context.until is not None:
            until = DT2dt(self.context.until)
            until = until.replace(hour=23, minute=59, second=59, microsecond=999999)
        else:
            until = None
        # Make it end at the end of the day:
        indizies = [int(i) for i in self.context.getWeekdays()]
        days = []
        bysetpos = None
        for i in indizies:
            days.append(wdays[i])
        
        if self.context.getBysetpos():
            bysetpos = [int(i) for i in self.context.getBysetpos().split(',')] 
                
        rule = rrule.rrule(self.context.frequency,
                       dtstart=dtstart,
                       interval=self.context.interval,
                       #wkst=None, 
                       count=self.context.count, 
                       until=until, 
                       # byweekday=days,
                       # bysetpos=self.context.getBysetpos(),
                       #bysetpos=None,
                       #bymonth=None, bymonthday=None, byyearday=None, byeaster=None,
                       #byweekno=None, byweekday=None,
                       #byhour=None, byminute=None, bysecond=None,
                       #cache=False
                   )
        if days and bysetpos:
            rule = rrule.rrule(self.context.frequency,
                           dtstart=dtstart,
                           interval=self.context.interval,
                           #wkst=None, 
                           count=self.context.count, 
                           until=until, 
                           byweekday=days,
                           bysetpos=bysetpos,
                           #bysetpos=None,
                           #bymonth=None, bymonthday=None, byyearday=None, byeaster=None,
                           #byweekno=None, byweekday=None,
                           #byhour=None, byminute=None, bysecond=None,
                           #cache=False
                       )
        
        
        return rule

    def getOccurrenceDays(self, until=None):
        """Days on which the event occurs. Used for indexing"""
        # XXX Handle when there is no occurrence.
        rule = self.getRecurrenceRule()
        if rule is None:
            return []
        if until is None:
            until = datetime.datetime.now() + \
                    datetime.timedelta(365*5)

        if until.tzinfo is None and rule._dtstart.tzinfo is not None:
            until = until.replace(tzinfo=rule._dtstart.tzinfo)
            
        if until.tzinfo is not None and rule._dtstart.tzinfo is None:
            until = until.replace(tzinfo=None)
                            
        if rule._until is None or rule._until > until:
            rule._until = until
        
        to_exclude = self.context.getExceptions()
        return [x.date().toordinal() for x in rule if x.date().strftime('%Y-%m-%d') not in to_exclude ][1:]


class EventRecurrenceConfig(object):
    """An IRecurrenceConfig adapter for events.
    """
    
    interface.implements(IRecurrenceConfig)
    component.adapts(VSEvent)

    def __init__(self, context):
        self.context = context

    def __get_is_recurring(self):
        return kalends.IRecurringEvent.providedBy(self.context) and \
               annointerfaces.IAttributeAnnotatable.providedBy(self.context)
    def __set_is_recurring(self, activated):
        ifaces = interface.directlyProvidedBy(self.context)
        if activated:
            if not kalends.IRecurringEvent.providedBy(self.context):
                ifaces += kalends.IRecurringEvent
            if not annointerfaces.IAttributeAnnotatable.providedBy(self.context):
                ifaces += annointerfaces.IAttributeAnnotatable
            if getattr(self.context, 'layout', None) is not None:
                self.context.layout = 'month.html'
        else:
            if kalends.IRecurringEvent in ifaces:
                ifaces -= kalends.IRecurringEvent
            if getattr(self.context, 'layout', None) is not None:
                delattr(self.context, 'layout')
        interface.directlyProvides(self.context, ifaces)

    is_recurring = property(__get_is_recurring,
                            __set_is_recurring)



################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther
################################################################

from plone.memoize.instance import memoize
from datetime import date 
from dateable.kalends import IRecurrence
from dateutil.parser import parse
from zope.i18n import translate
from Globals import InitializeClass
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from AccessControl import getSecurityManager
from Products.CMFCore.permissions import ModifyPortalContent, View
from dateable.kalends import IRecurringEvent

from vs.event.config import *
from vs.event.interfaces import IVSSubEvent

FREQ = {0: 'year',
        1: 'month',
        2: 'week',
        3: 'day',
        4: 'hour',
        5: 'minute',
        6: 'second',
    }

class VSEventView(BrowserView):
    """ vs_event_view browser view """

    def same_day(self):
        return self.context.start().Date() == self.context.end().Date()

    def short_start_date(self):
        return self.context.toLocalizedTime(self.context.start(), long_format=0)
        
    def long_start_date(self):
        return self.context.toLocalizedTime(self.context.start(), long_format=1)
    
    def start_time(self):
        return self.context.start().strftime(self.time_format())

    def short_end_date(self):
        return self.context.toLocalizedTime(self.context.end(), long_format=0)
    
    def long_end_date(self):
        return self.context.toLocalizedTime(self.context.end(), long_format=1)

    def end_time(self):
        return self.context.end().strftime(self.time_format())

    def datetime_format(self):
        site_properties = self.context.portal_properties.site_properties
        return site_properties.getProperty('localLongTimeFormat')

    def date_format(self):
        site_properties = self.context.portal_properties.site_properties
        return site_properties.getProperty('localTimeFormat')
    
    def time_format(self):
        datetime_format = self.datetime_format()
        if '%p' in datetime_format:
            # Using AM/PM:
            return ' '.join(datetime_format.split(' ')[-2:])
        # 24 H format
        return datetime_format.split(' ')[-1]

    def isRecurring(self):
        return IRecurringEvent.providedBy(self.context)

    def rrule(self):
        return IRecurrence(self.context, None).getRecurrenceRule()

    def rrule_freq(self):
        rrule = self.rrule()
        if rrule is None:
            return ''
        if rrule._interval == 1:
            text = u"Every ${frequency}"
        else:
            text = u"Every ${interval} ${frequency}s"

        return translate(text, mapping={'interval':rrule._interval, 
                                        'frequency':FREQ[rrule._freq]})
    def rrule_interval(self):
        rrule = self.rrule()
        if rrule is not None:
            return rrule._interval
        return 0
        
    def rrule_end(self):
        rrule = self.rrule()
        if rrule is not None and rrule._until:
            return self.context.toLocalizedTime(dt2DT(rrule._until), long_format=0)
        return ''
    @memoize    
    def filteredAttendees(self):
        """ return list of attendees with 'show' flag set """
        attendees = self.context.getAttendees()
        result = []
        for attendee in attendees:
            if attendee['show']:
                result.append(attendee)
        return result


    def getMainEvent(self):
        """ return the main event from the list of backrefs """

        backrefs = self.context.getBRefs()
        if backrefs:
            return backrefs[0]
        return None

    def getSupplementaryEvents(self):
        """ return list of suppl. events in sorted order """

        sm = getSecurityManager()
        events = self.context.getSubEvents()
        events = [ev for ev in events if sm.checkPermission(View, ev)]
        events.sort(lambda e1,e2: cmp(e1.start(), e2.start()))
        return events

    @memoize
    def nextDates(self):
        """ Recurrence """
        return [ date.fromordinal(x) for x in IRecurrence(self.context, None).getOccurrenceDays()]
 
    @memoize
    def getExceptions(self):
        """ get recurrence exceptions """
        return [parse(x) for x in self.context.getExceptions()]


    @memoize
    def toLocalizedTime(self, time, long_format=None):
        """Convert time to localized time
        """
        properties=getToolByName(self.context, 'portal_properties').site_properties
        if long_format:
            format=properties.localLongTimeFormat
        else:
            format=properties.localTimeFormat

        return time.strftime(format)

    @memoize
    def allowedToAddSubEvents(self):
        """ Security chess """
        return getSecurityManager().checkPermission(ModifyPortalContent, self)


    @memoize
    def subeventsEnabled(self):
        """Convert time to localized time
        """
        calendar = getToolByName(self.context, 'portal_calendar')
        return getattr(calendar, 'vs_event_supplementary_events', False)


    @memoize
    def isSubEvent(self):
        """ check if the current object is a VSSubEvent """
        return IVSSubEvent.providedBy(self.context)


    def getAttachments(self):
        """ Return all viewable attachments """
        if self.isSubEvent():
            return ()
        mtool = self.context.portal_membership
        result = []
        objs = self.context.getAttachments()
        return [o for o in objs if mtool.checkPermission('View', o)]

InitializeClass(VSEventView)



# -*- coding: utf-8 -*-
################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther 
################################################################

from zope.i18nmessageid import MessageFactory
from config import PROJECTNAME

from Products.CMFCore import utils
from Products.Archetypes import atapi

MessageFactory = MessageFactory(PROJECTNAME)
from config import PROJECTNAME
from logging import getLogger
log = getLogger(">>>>>>>>")

def initialize(context):
    """Initializer called when used as a Zope 2 product."""

    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(PROJECTNAME),
        PROJECTNAME)

    ADD_PERMISSIONS={}
    for type in content_types:
        ADD_PERMISSIONS[type.portal_type] = """Add %s""" %type.portal_type

    for atype, constructor in zip(content_types, constructors):
        utils.ContentInit("%s: %s" % (PROJECTNAME, atype.portal_type),
        content_types      = (atype,),
        permission         = ADD_PERMISSIONS[atype.portal_type],
        extra_constructors = (constructor,),
        ).initialize(context)


#from Products.CMFCore.permissions import setDefaultRoles
#setDefaultRoles('Add VSEvent', ('Manager', ))



################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther 
################################################################

import re
from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator
from Products.validation.validators.RegexValidator import RegexValidator

class VSLinesOfDateValidator:
    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
        self.re_line = re.compile('^\d{4}-\d{2}-\d{2}$')

    def __call__(self, value, *args, **kwargs):
        error_msg = ""
        for val in value:
            if not self.re_line.match(val):
                error_msg = "invalid date-string"

        if error_msg:
            return error_msg
        else:
            return True

validators = [
    RegexValidator(
        'isLineOfInts',
        r'^$|-?\d+(,-?\d+)*$',
        title='',
        description='',
        errmsg='',
    ),
    VSLinesOfDateValidator(
        'linesOfDates', title="Validator for Date-Strings",
        description=""
    ),
]

for validator in validators:
    validation.register(validator)
         



from dateable import kalends
from zope.component import ComponentLookupError
from Products.CMFPlone.CatalogTool import registerIndexableAttribute

def recurrence_days(object, portal, **kwargs):
    """Return the dates of recurrences as ordinals
    """
    try:
        recurrence = kalends.IRecurrence(object)
        return recurrence.getOccurrenceDays()
    except (ComponentLookupError, TypeError, ValueError):
        # The catalog expects AttributeErrors when a value can't be found
        raise AttributeError

registerIndexableAttribute('recurrence_days', recurrence_days)



################################################################
# vs.event - published under the GPL 2
# Authors: Andreas Jung, Veit Schiele, Anne Walther
################################################################

from dateable import kalends
import datetime
from dateutil import rrule

import zope.interface
from Testing import ZopeTestCase
from Products.Five import zcml, fiveconfigure 
from AccessControl import getSecurityManager
from DateTime.DateTime import DateTime
from Products.PloneTestCase import PloneTestCase
from Products.PloneTestCase import layer
from Products.PloneTestCase.layer import onsetup

from vs.event.content import event_util

PloneTestCase.installProduct('vs.event')

@onsetup
def setup_package():
    fiveconfigure.debug_mode = True
    import vs.event
    zcml.load_config('configure.zcml', vs.event)
    fiveconfigure.debug_mode = False
    ZopeTestCase.installPackage('vs.event')

setup_package()
PloneTestCase.setupPloneSite(products=('vs.event',))

class TestBase(PloneTestCase.FunctionalTestCase):

    def afterSetUp(self):
        ZopeTestCase.utils.setupCoreSessions(self.app)
        self.membership = self.portal.portal_membership
        self.membership.addMember('god', 'secret', ['Manager'], [])

class VSEventTest(TestBase):

    def testCalendarToolCheck(self):
        pc = self.portal.portal_calendar
        self.assertEqual(pc.meta_type, 'Chronos Calendar Tool')

    def testProperties(self):
        pc = self.portal.portal_calendar
        self.assertEqual(pc.vs_event_supplementary_events, True)
        self.assertEqual('VSEvent' in pc.getCalendarTypes(), True)
        self.assertEqual('VSSubEvent' in pc.getCalendarTypes(), True)

    def testCreateEvent(self):
        self.login('god')
        user = getSecurityManager().getUser()
        self.assertEqual('Manager' in user.getRoles(), True)
        self.portal.invokeFactory('VSEvent', id='foo')
        event = self.portal['foo']
        self.assertEqual(event.portal_type, 'VSEvent') 

    def testOneDayEvent(self):
        self.login('god')
        self.portal.invokeFactory('VSEvent', id='foo')
        event = self.portal['foo']
        event.setStartDate(DateTime(2009, 01, 01))
        event.setEndDate(DateTime(2009, 01, 01))
        event.setWholeDay(True)
        d = event_util.date_for_display(event)
        self.assertEqual(d['from_str'], 'Jan 01, 2009')
        self.assertEqual(d['to_str'], None)
        self.assertEqual(d['same_day'], True)

    def testSeveralDaysEvent(self):
        self.login('god')
        self.portal.invokeFactory('VSEvent', id='foo')
        event = self.portal['foo']
        event.setStartDate(DateTime(2009, 01, 01))
        event.setEndDate(DateTime(2009, 12, 31))
        event.setWholeDay(True)
        d = event_util.date_for_display(event)
        self.assertEqual(d['from_str'], 'Jan 01, 2009')
        self.assertEqual(d['to_str'], 'Dec 31, 2009')
        self.assertEqual(d['same_day'], False)

    def testiCal(self):
        self.login('god')
        self.portal.invokeFactory('VSEvent', id='foo')
        event = self.portal['foo']
        data = event.getICal()

    def testVCal(self):
        self.login('god')
        self.portal.invokeFactory('VSEvent', id='foo')
        event = self.portal['foo']
        data = event.getVCal()

class RecurrenceTest(TestBase):

    def testRecurranceBasic(self):
        self.login('god')
        # Basic recurrence, daily for one year:
        self.folder.invokeFactory('VSEvent', 'event')
        event = getattr(self.folder, 'event')
        event.update(startDate = DateTime('2001/02/01 10:00'),
                     endDate = DateTime('2001/02/01 14:00'))

        # Mark as recurring
        zope.interface.alsoProvides(event, kalends.IRecurringEvent)
        recurrence = kalends.IRecurrence(event)

        # Set the recurrence info
        event.frequency=rrule.DAILY
        event.until=DateTime('2002/02/01')
        event.interval = 1
        event.count = None

        # Test
        dates = recurrence.getOccurrenceDays()
        self.failUnlessEqual(dates[0], datetime.date(2001, 2, 2).toordinal())
        self.failUnlessEqual(dates[-1], datetime.date(2002, 2, 1).toordinal())
        self.failUnlessEqual(len(dates), 365)

        # Try with an interval
        event.interval = 3
        dates = recurrence.getOccurrenceDays()
        self.failUnlessEqual(dates[0], datetime.date(2001, 2, 4).toordinal())
        self.failUnlessEqual(dates[-1], datetime.date(2002, 1, 30).toordinal())
        self.failUnlessEqual(len(dates), 121)

        # Have a max count:
        event.count = 25
        dates = recurrence.getOccurrenceDays()
        self.failUnlessEqual(len(dates), 24)

    def testRecurranceMidnight(self):
        # Check that the recurrence works correctly with events starting
        # at midnight
        self.login('god')
        self.folder.invokeFactory('VSEvent', 'event')
        event = getattr(self.folder, 'event')

        event.update(startDate = DateTime('2001/02/01 00:00'),
                     endDate = DateTime('2001/02/01 04:00'))

        # Mark as recurring
        zope.interface.alsoProvides(event, kalends.IRecurringEvent)
        recurrence = kalends.IRecurrence(event)

        # Set the recurrence info
        event.frequency=rrule.DAILY
        event.until=DateTime('2001/02/04')
        event.interval=1
        event.count=None

        # Test
        dates = recurrence.getOccurrenceDays()        
        self.failUnlessEqual(dates[0], datetime.date(2001, 2, 2).toordinal())
        self.failUnlessEqual(dates[-1], datetime.date(2001, 2, 4).toordinal())
        self.failUnlessEqual(len(dates), 3)

    def testRecurranceWeek(self):
        self.login('god')
        self.folder.invokeFactory('VSEvent', 'event')
        event = getattr(self.folder, 'event')

        event.update(startDate = DateTime('2007/02/01 00:00'),
                     endDate = DateTime('2007/02/01 04:00'))

        # Mark as recurring
        zope.interface.alsoProvides(event, kalends.IRecurringEvent)
        recurrence = kalends.IRecurrence(event)

        # Set the recurrence info
        event.frequency=rrule.WEEKLY
        event.until=DateTime('2008/02/04')
        event.interval=1
        event.count=None

        # Test
        dates = recurrence.getOccurrenceDays()
        self.failUnlessEqual(dates[0], datetime.date(2007, 2, 8).toordinal())
        self.failUnlessEqual(dates[1], datetime.date(2007, 2, 15).toordinal())
        self.failUnlessEqual(dates[2], datetime.date(2007, 2, 22).toordinal())
        self.failUnlessEqual(dates[-1], datetime.date(2008, 1, 31).toordinal())
        self.failUnlessEqual(len(dates), 52)


def test_suite():
    from unittest import TestSuite, makeSuite

    suite = TestSuite()
    suite.addTests(makeSuite(VSEventTest))
    suite.addTests(makeSuite(RecurrenceTest))
    suite.layer = layer.ZCMLLayer
    return suite



# -*- coding: utf-8 -*-
# $Id: event.py 388 2009-06-13 09:32:31Z ajung $



from AccessControl import ClassSecurityInfo
from zope.interface import implements

from DateTime.DateTime import DateTime
from Products.ATContentTypes.content.event import ATEvent
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn

from Products.CMFCore.permissions import View
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from dateutil.rrule import YEARLY, MONTHLY, WEEKLY, DAILY

from vs.event.config import *
from vs.event.interfaces import IVSEvent, IVSSubEvent
from vs.event import MessageFactory as _
from vs.event import validators 
from vs.event.fieldsandwidgets.calendarwidget import VSCalendarWidget
import event_util

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    from Products.Archetypes import atapi


from logging import getLogger
log = getLogger(">> event : ")

VSEventSchema = atapi.Schema((

    atapi.ReferenceField(
        name='subEvents',
        allowed_types=('VSSubEvent',),
        multiValued=True,
        relationship='isSubEvent',
        widget=ReferenceBrowserWidget(
            visible={'view': 'invisible', 'edit':'invisible'},
        ),
    ), 
    atapi.LinesField(
        name='weekdays',
        schemata='recurrence',
        vocabulary=atapi.DisplayList((
            ('0', _(u'vs_label_mo')),
            ('1', _(u'vs_label_di')),
            ('2', _(u'vs_label_mi')),
            ('3', _(u'vs_label_do')),
            ('4', _(u'vs_label_fr')),
            ('5', _(u'vs_label_sa')),
            ('6', _(u'vs_label_so')) 
        )),
        widget=atapi.MultiSelectionWidget(
            format="checkbox",
            label=_(u'vs_event_label_weekdays',),
            description=_(u'vs_event_help_weekdays'),
        ),
    ),
    atapi.StringField(
        name='bysetpos',
        schemata='recurrence',
        validators=('isLineOfInts',),
        widget=atapi.StringWidget(
            label=_(u'vs_event_label_bysetpos'),
            description=_(u'vs_event_help_bysetpos'),
        ),
    ),
    atapi.LinesField(
        name='exceptions',
        schemata='recurrence',
        validators=('linesOfDates',),
        widget=atapi.LinesWidget(
            label=_(u'vs_event_label_exceptions'),
            description=_(u'vs_event_help_exceptions'),
        ),
    ),

    atapi.IntegerField('frequency',
         schemata="recurrence",
         required=True,
         vocabulary={-1: u'Does not repeat',
                     YEARLY: u'Yearly',
                     MONTHLY: u'Monthly',
                     WEEKLY: u'Weekly',
                     DAILY: u'Daily',
                     }.items(),
         default=-1,
         widget=atapi.SelectionWidget(label=u'Repeats')
         ),
    atapi.IntegerField('interval',
         schemata="recurrence",
         required=True,
         default=1,
         widget=atapi.IntegerWidget(label=u'Repeats every',
              description=u"Repeats every day/week/month/year.")
         ),
    atapi.DateTimeField('until',
         schemata="recurrence",
         widget=VSCalendarWidget(description=u"Event repeats until this date.",
                                 description_msgid = "vs_repeat_events_until_date",
                                 label="Repeat until",
                                 label_msgid="vs_label_repeat_until",
                                 i18n_domain="vs.event",
                                 with_time=1),
         ),
    atapi.IntegerField('count',
          schemata="recurrence",
          widget=atapi.IntegerWidget(label=u'Count',
                                     description=u'Maxinum number of times the event repeats ',)
          ),

))

VSEventSchema = VSEventSchema + ATEvent.schema.copy()
finalizeATCTSchema(VSEventSchema, folderish=False, moveDiscussion=True)


def modifyEventSchema(schema):
    schema.addField(atapi.BooleanField('wholeDay',
                                       description_msg='help_whole_day_event',
                                       widget=atapi.BooleanWidget(
                                           label='Whole day event',
                                           label_msgid='label_whole_day_event',
                                           i18n_domain='vs.event'),
                                       ))
    schema.addField(atapi.BooleanField('useEndDate',
                                       default=True,
                                       description_msg='help_has_end_date',
                                       widget=atapi.BooleanWidget(
                                           label='useEndDate',
                                           label_msgid='label_use_end_date',
                                           i18n_domain='vs.event'),
                                       ))
    schema.moveField('wholeDay', before='startDate')
    schema.moveField('useEndDate', after='wholeDay')
    schema['startDate'].widget = VSCalendarWidget(description= "",
                                                description_msgid = "help_event_start",
                                                label="Event Starts",
                                                label_msgid = "label_event_start",
                                                i18n_domain="vs.event",
                                                with_time=1,
                                                with_popup=1,
                                                js_shortcuts=0,
                                                ignore_unset_time=1)

    schema['endDate'].widget = VSCalendarWidget(description = "",
                                              description_msgid = "help_event_end",
                                              label = "Event Ends",
                                              label_msgid = "label_event_end",
                                              i18n_domain = "vs.event",
                                              with_time=1,
                                              with_popup=1,
                                              js_shortcuts=0,
                                              ignore_unset_time=1)

    del schema['until']
    schema.addField(atapi.DateTimeField('until',
                                 schemata='recurrence',
                                 widget=VSCalendarWidget(description=u"Event repeats until this date",
                                 description_msgid="vs_repeat_events_until_date",
                                 label="Repeat until",
                                 label_msgid="vs_label_repeat_until",
                                 i18n_domain="vs.event",
                                 with_time=1)))

    schema.addField(atapi.ReferenceField(name='attachments',
                                allowed_types=('Link','File'),
                                multiValued=1,
                                relationship='aAttachment',
                                widget=ReferenceBrowserWidget(
                                    show_review_state=1,
                                    allow_sorting=1,
                                    force_close_on_insert=1,
                                    base_query = {'Type':['Link','File']},
                                    label=_(u'vs_event_label_attachments',),
                                ),
                            )) 

    del schema['attendees']
    schema.addField(DataGridField(name="attendees",
                            columns=('name', 'mail','role', 'show'),
                            schemata='attendees',
                            widget = DataGridWidget(
                                label=_(u'vs_event_label_roleAttendees',),
                                columns={
                                    'name':     Column(_(u'vs_event_label_nameColumn')),
                                    'mail':     Column(_(u'vs_event_label_mailColumn')),
                                    'role':     SelectColumn(_(u'vs_event_label_roleColumn'), vocabulary='getAttendeeRoles'),
                                    'show':     CheckboxColumn(_(u'vs_event_label_showColumn')),
                                }),
                            ))
    return schema

class VSEvent(ATEvent):
    """ vs.event """

    security = ClassSecurityInfo()
    implements(IVSEvent)
    meta_type = 'VSEvent'
    _at_rename_after_creation = True
    schema = modifyEventSchema(VSEventSchema)

    security.declareProtected(View, 'getICal')
    def getICal(self):
        """get iCal data """
        return event_util.getICal(self)

    security.declareProtected(View, 'getVCal')
    def getVCal(self):
        """get VCal data """
        return event_util.getVCal(self)


    def at_post_edit_script(self):
        """ Ensure that for single-day dates without an end date
            the end date is equal to the start date.
        """

        self.setExcludeFromNav(True)

        if not self.getUseEndDate():
            self.setEndDate(self.start())

        if self.getWholeDay():
            self.setStartDate(DateTime(self.start().strftime('%Y/%m/%d 00:00:00')))
            self.setEndDate(DateTime(self.end().strftime('%Y/%m/%d 23:59:00')))

    def getAttendeeRoles(self):
        """ """
        return atapi.DisplayList((
            ('chair',_(u'vs_event_label_chair')),
            ('observer',_(u'vs_event_label_observer')),
            ('participant',_(u'vs_event_label_participant')),
            ('opt_participant',_(u'vs_event_label_opt_participant')), 
        )) 

    security.declareProtected(View, 'post_validate')
    def post_validate(self, REQUEST=None, errors=None):
        """ Trigger original ATEvent.post_validate_method() for
            dates having an end date.
        """
        if REQUEST.get('useEndDate', True) == True:
            return super(VSEvent, self).post_validate(REQUEST=REQUEST, 
                                                      errors=errors)

atapi.registerType(VSEvent, PROJECTNAME)


def modifySubEventSchema(schema):
    # remove unwanted fields for subevents
    for id in ('attendees', 'contactName', 'contactEmail', 'contactPhone', 'eventType', 'eventUrl'):
        schema[id].widget.visible = False
    for field in schema.fields():
        if field.schemata in ('dates', 'categorization', 'ownership', 'settings'):
            field.widget.visible = False

    schema.addField(atapi.BooleanField('wholeDay',
                                       default=False,
                                       description_msg='help_whole_day_event',
                                       label='Whole day event',
                                       label_msgid='label_whole_day_event',
                                       i18n_domain='vs.event',
                                       ))
    schema.addField(atapi.BooleanField('useEndDate',
                                       default=True,
                                       description_msg='help_has_end_date',
                                       label='useEndDate',
                                       label_msgid='label_use_end_date',
                                       i18n_domain='vs.event',
                                       ))
    schema.moveField('wholeDay', before='startDate')
    schema.moveField('useEndDate', after='wholeDay')

    schema['startDate'].widget = VSCalendarWidget(description= "",
                                                description_msgid = "help_event_start",
                                                label="Event Starts",
                                                label_msgid = "label_event_start",
                                                i18n_domain = "vs.event",
                                                with_time=1,
                                                with_popup=1,
                                                js_shortcuts=0,
                                                ignore_unset_time=1)

    schema['endDate'].widget = VSCalendarWidget(description = "",
                                              description_msgid = "help_event_end",
                                              label = "Event Ends",
                                              label_msgid = "label_event_end",
                                              i18n_domain = "vs.event",
                                              with_time=1,
                                              with_popup=1,
                                              js_shortcuts=0,
                                              ignore_unset_time=1)

    return schema

VSSubEventSchema = ATEvent.schema.copy()
finalizeATCTSchema(VSSubEventSchema, folderish=False, moveDiscussion=True)

class VSSubEvent(VSEvent):
    """ vs.event """

    implements(IVSSubEvent)
    meta_type = 'VSSubEvent'
    schema = modifySubEventSchema(VSSubEventSchema)

atapi.registerType(VSSubEvent, PROJECTNAME)



from setuptools import setup, find_packages
import os

version = '0.2.15'

setup(name='vs.event',
      version=version,
      description="An extended event content-type for Plone (and Plone4Artists calendar)",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='Zope Plone Event Recurrence Calendar Plone4Artists',
      author='Veit Schiele, Anne Walther, Andreas Jung',
      author_email='vs.event@veit-schiele.de',
      url='http://svn.plone.org/svn/collective/vs.event',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['vs'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'python-dateutil',
          'dateable.chronos',
          'dateable.kalends',
          'Products.DataGridField',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
